;;; eev-code.el -- `code-c-d' and related functions, that generate
;;; Lisp code with definitions and evaluate it. Note: this file
;;; contains protype-ish code, that is intented to replace some parts
;;; of eev in a near future - BUT THIS IS NOT YET LOADED BY DEFAULT!

;; Version: 2012sep30

;; (code-brfile 'find-fline                  :local 'brfl)
;; (code-brfile 'find-xpdfpage               :local 'brxpdf :dired 'brxpdfd)
;; (code-brurl  'find-firefox  :remote 'brm  :local 'brml   :dired 'brmd)

;; (find-elnode "Symbol Type" "`:'")
;; (find-elnode "Constant Variables")
;; (find-node "(cl)Argument Lists")
;; (find-node "(cl)Argument Lists" "&body")
;;
(defun ee-tail-call (fmt rest)
  "An internal function used to support keyword-argument pairs."
  (cond ((null rest) "")
	((keywordp (car rest))
	 (apply (intern (format fmt (car rest)))
		(cdr rest)))
	(t (error "Wrong rest: %S" rest))))

(defun ee-read (str) (read (concat "(progn\n" str "\n)")))



;;;        _ _     _       
;;;   __ _| (_)___| |_ ___ 
;;;  / _` | | / __| __/ __|
;;; | (_| | | \__ \ |_\__ \
;;;  \__,_|_|_|___/\__|___/
;;;                        

(defun ee-aref (alist idx)
  "Like `aref', but for alists.
Example: (ee-aref '((1 . one) (2 . two) (3 . three)) 2)
                                -> two"
  (cdr (assoc idx alist)))

(defun ee-adel (alist idx)
  "Like `remq', but for alists. This is non-destructive, so wrap it in a setq.
Example: (ee-adel '((1 . one) (2 . two) (3 . three)) 2)
                -> ((1 . one)           (3 . three))"
  (remq (assoc idx alist) alist))

(defun ee-aset (alist idx newelt)
  "Like `aset', but for alists. This is non-destructive, so wrap it in a setq.
Example: (ee-aset '((1 . one) (2 . two) (3 . three)) 2 'foo)
      -> ((2 . foo) (1 . one)           (3 . three))"
  (cons (cons idx newelt) (ee-adel alist idx)))

(defun ee-areplace (alist idx newelt)
  "Like `ee-aset', but keeping the order.
Examples: (ee-areplace '((1 . one) (2 . two) (3 . three)) 2 'foo)
                     -> ((1 . one) (2 . foo) (3 . three))
          (ee-areplace '((1 . one) (2 . two) (3 . three)) 0 'zero)
          -> ((0 . zero) (1 . one) (2 . two) (3 . three))"
  (if (ee-aref alist idx)
      (progn (setcdr (assoc idx alist) newelt)
	     alist)
    (cons (cons idx newelt) alist)))

;;;                _                          _                   _          
;;;   ___ ___   __| | ___        ___       __| |      _ __   __ _(_)_ __ ___ 
;;;  / __/ _ \ / _` |/ _ \_____ / __|____ / _` |_____| '_ \ / _` | | '__/ __|
;;; | (_| (_) | (_| |  __/_____| (_|_____| (_| |_____| |_) | (_| | | |  \__ \
;;;  \___\___/ \__,_|\___|      \___|     \__,_|     | .__/ \__,_|_|_|  |___/
;;;                                                  |_|                     
;; Old: (find-evardescr 'code-c-d-keywords)
;;      (find-evariable 'code-c-d-keywords)

(defvar ee-code-c-d-pairs nil
  "Each (code-c-d C D) call generates an entry (C (ee-expand D)) here.
A new entry with the same C as a previous one will replace the
previous one. This list is maintained by `ee-code-c-d-add-pair' and
is used by some functions in \"eev-insert.el\".")

(defun ee-code-c-d-add-pair (c d)
  (setq ee-code-c-d-pairs (ee-areplace ee-code-c-d-pairs c (list d))))





;;;                _                          _ 
;;;   ___ ___   __| | ___        ___       __| |
;;;  / __/ _ \ / _` |/ _ \_____ / __|____ / _` |
;;; | (_| (_) | (_| |  __/_____| (_|_____| (_| |
;;;  \___\___/ \__,_|\___|      \___|     \__,_|
;;;                                             
;; See: (find-code-c-d-intro)

;; code-c-d: top-level functions
;;
(defun      code-c-d (c d &rest rest)
  (ee-code-c-d-add-pair c d)
  (eval (ee-read (apply 'ee-code-c-d c d rest))))
(defun find-code-c-d (c d &rest rest)
  (find-estring-elisp (apply 'ee-code-c-d c d rest)))
(defun   ee-code-c-d (c d &rest rest)
  (if (stringp (car rest))
      (setq rest (cons :info rest)))
  (concat (ee-code-c-d-base c d)
	  (ee-code-c-d-rest rest)))

;; Support for extra arguments
;;
(defun ee-code-c-d-rest (rest)
  (ee-tail-call "ee-code-c-d-%S" rest))

(defun ee-code-c-d-base (c d)
  (ee-template0 "
   ;; See: (find-eev \"eev-code.el\")
   (setq ee-{c}dir \"{d}\")
   (setq ee-{c}tagsfile \"{d}TAGS\")
   (defun ee-{c}file (str)
     (concat (ee-expand ee-{c}dir) str))
   (defun ee-use-{c}-tags ()
     (setq tags-file-name ee-{c}tagsfile))
   (defun find-{c}file (str &rest pos-spec-list)
     (interactive (list \"\"))
     (ee-use-{c}-tags)
     (apply 'find-fline (ee-{c}file str) pos-spec-list))
   (defun find-{c}tag (str &rest pos-spec-list)
     (ee-use-{c}-tags)
     (apply 'ee-find-tag str pos-spec-list))
   (defun find-{c}sh (command &rest pos-spec-list)
     (apply 'ee-find-xxxsh ee-{c}dir command pos-spec-list))
   (defun find-{c}sh0 (command)
     (funcall 'ee-find-xxxsh0 ee-{c}dir command))
   (defun find-{c}sh00 (command)
     (funcall 'ee-find-xxxsh00 ee-{c}dir command))
   (defun find-{c}w3m (furl &rest pos-spec-list)
     (apply 'find-w3m (ee-{c}file furl) pos-spec-list))
   (defun find-{c}grep (grep-command-args &rest pos-spec-list)
     (apply 'ee-find-grep ee-{c}dir grep-command-args pos-spec-list))
   "))

(defun ee-code-c-d-:info (info &rest rest)
  (concat (ee-template0 "
   (defun find-{c}node (page &rest pos-spec-list)
     (interactive (list \"\"))
     (setq ee-info-code \"{c}\")
     (setq ee-info-file \"{info}\")    ;; a (temporary) hack for M-h M-i
     (apply 'find-node (format \"({info})%s\" page) pos-spec-list))
   ") (ee-code-c-d-rest rest)))

(defun ee-code-c-d-:linfo (manual &rest rest)
  (concat (ee-template0 "
   (defun find-{c}node (section &rest pos-spec-list)
     (interactive (list \"\"))
     (apply 'ee-find-node ee-{c}dir \"{manual}\" section pos-spec-list))
   ") (ee-code-c-d-rest rest)))

(defun ee-code-c-d-:anchor (&rest rest)
 (concat (ee-template0 "
   (defun find-{c} (str &rest pos-spec-list)
     (apply 'find-anchor (ee-{c}file str) pos-spec-list))
   ") (ee-code-c-d-rest rest)))

(defun ee-code-c-d-:wget (url &rest rest)
 (concat (ee-template0 "
   (defun ee-{c}url (semiurl) (concat \"{url}\" semiurl))
   (defun find-{c}wget (semiurl &rest pos-spec-list)
     (interactive (list \"\"))
     (apply 'find-wget (ee-{c}url semiurl) pos-spec-list))
   ") (ee-code-c-d-rest rest)))

(defun ee-code-c-d-:grep (&rest rest) (ee-code-c-d-rest rest))  ; compat

;; support functions
;;
(defun ee-find-node (dir manual page &rest pos-spec-list)
  (apply 'find-node (format "(%s%s)%s" dir manual page) pos-spec-list))

(defun ee-find-grep (dir grep-command-args &rest pos-spec-list)
  "Example: (ee-find-grep ee-eetcdir \"grep -niH -e tetris *\")
Note: the POS-SPEC-LIST arguments are currently not used."
  (let ((default-directory (ee-expand (or dir default-directory))))
    (grep grep-command-args)))


;; a test
;; (find-estring-elisp (ee-code-c-d-base "@@@" "!!!"))
;; (find-estring-elisp (ee-code-c-d "CCC" "DDD"))







;; (find-efunction-links 'eeurl-u-to-u-l)
;; (find-efunction       'eeurl-u-to-u-l)
;; (find-efunction-links 'eeurl-f-to-u)
;; (find-efunction       'eeurl-f-to-u)
;; (find-efunction-links 'eeurl-dired-file-name-at-point)
;; (find-efunction       'eeurl-dired-file-name-at-point)








;; (find-efunction 'code-c-d-new)

;; (find-efunction 'eeurl-dired-file-name-at-point)
;; ee-dired-file-name-at-point
;; ee-url-to-local-file
;; ee-url-to-local-url
;; ee-url-to-local-file-url
;; ee-file-to-fileurl
;; ee-url-to-fileurl
;; snarffname
;; snarfurl

;; (find-efunctionpp 'brm)
;; (find-code-brurl 'browse-url-firefox :remote: 'brm)
;; (find-efunctionpp 'brml)
;; (find-efunctionpp 'brmd)

;; (find-efunctionpp 'find-djvupage)
;; (find-efunctionpp 'brdjvul)
;; (find-efunctionpp 'brdjvud)
;; (find-efunctionpp 'browse-url-firefox)
;; (find-efunctionpp 'brm)
;; (find-efunctionpp 'brml)
;; (find-efunctionpp 'brmd)

;; (find-efunction 'ee-template)
;; (find-elnode "Index" "* defvaralias:")
;; ee-hp

;; (find-efunctionpp 'brm)
;; (find-efunctionpp 'brml)
;; (find-efunctionpp 'brmd)
;; (find-efunction 'ee-concat)

;; (format "%s" 'foo)
;; (format "%s" "foo")
;; (format "%s" 123)
;; (ee-intern "foo%s" 123)

;; Internal functions (generic)

;; (defun ee-intern (string &rest objects)
;;   "The result of (format STRING OBJECTS), converted to a symbol"
;;   (intern (apply 'format string objects)))



(provide 'eev-code)




;; Local Variables:
;; coding:            raw-text-unix
;; ee-anchor-format:  "%s"
;; ee-anchor-format:  "defun %s "
;; no-byte-compile:   t
;; End:
