;;; eev.el -- variants of eval-last-sexp.

;; Copyright (C) 2012 Free Software Foundation, Inc.
;;
;; This file is not yet part of GNU eev.


(require 'eev-flash)		; (find-eev "eev-flash.el")



;;;                  _                                            _ 
;;;   _____   ____ _| |      ___  _____  ___ __         ___  ___ | |
;;;  / _ \ \ / / _` | |_____/ __|/ _ \ \/ / '_ \ _____ / _ \/ _ \| |
;;; |  __/\ V / (_| | |_____\__ \  __/>  <| |_) |_____|  __/ (_) | |
;;;  \___| \_/ \__,_|_|     |___/\___/_/\_\ .__/       \___|\___/|_|
;;;                                       |_|                       
;;;
;;; evaluating sexps (alternatives to eval-last-sexp)
;;;

;; ee-eval-sexp-eol may be obsolete
;; ee-arg is still used in eev-insert.el (ack!)

;; See (find-efunction 'eval-last-sexp-1)
(defun ee-backward-sexp ()
  "An internal function used by `ee-eval-last-sexp'."
  (with-syntax-table emacs-lisp-mode-syntax-table
    (forward-sexp -1)
    (when (eq (preceding-char) ?\\)
      (forward-char -1)
      (when (eq (preceding-char) ??)
	(forward-char -1))))
  (point))

(defun ee-forward-sexp ()
  "An internal function used by `ee-eval-last-sexp'."
  (with-syntax-table emacs-lisp-mode-syntax-table
    (forward-sexp 1))
  (point))

(defun ee-last-sexp ()
  "An internal function used by `ee-eval-last-sexp'."
  (save-excursion
    (buffer-substring-no-properties
     (ee-backward-sexp) (ee-forward-sexp))))

(defmacro ee-no-debug (&rest body)
  `(let ((debug-on-error nil)) ,@body))

(defun ee-eval (sexp)
  (ee-no-debug (eval sexp)))

(defun ee-eval-last-sexp (&optional arg)
  (interactive "P")
  (cond ((eq arg 0)
	 (save-excursion
	   (eeflash+ (ee-backward-sexp) (ee-forward-sexp)
		     ee-highlight-spec)))
	((eq arg 1) (prin1 (ee-last-sexp)))
	((eq arg 2) (prin1 (read (ee-last-sexp))))
	;; ((eq arg 3) (ee-eval (read (ee-last-sexp))))
	((eq arg 3) (find-wset "13o_" ' (ee-eval-last-sexp)))
	((eq arg 4) (let ((sexp (read (ee-last-sexp)))) (debug) (eval sexp)))
	((eq arg 5) (let ((sexp (read (ee-last-sexp)))
			  (debug-on-error t))
		      (eval sexp)))
	((eq arg 8) (find-epp (ee-eval (read (ee-last-sexp)))))
	((eq arg 9) (let ((interactive-clause (read (ee-last-sexp))))
		      (let ((debug-on-error nil))
			(call-interactively
			 `(lambda (&rest args) ,interactive-clause
			    (message "%S" args))))))
	(t (prin1 (let ((ee-arg arg))
		    (ee-eval (read (ee-last-sexp))))))))

(defun ee-eval-sexp-eol (&optional arg)
  (interactive "P")
  (end-of-line)
  (ee-eval-last-sexp arg))



(provide 'eev-eval)





;; Local Variables:
;; coding:            raw-text-unix
;; ee-anchor-format:  "%s"
;; ee-anchor-format:  "defun %s "
;; ee-comment-prefix: ";;"
;; no-byte-compile:   t
;; End:
