;; eev-multiwindow.el - hacks for using eepitch with multi-window
;; setups (where by "multi" we means "more than two").
;;
;; Source:        <http://angg.twu.net/eev-current/eev-multiwindow.el>
;; Htmlized:      <http://angg.twu.net/eev-current/eev-multiwindow.el.html>
;; Documentation: <http://angg.twu.net/eev-current/eepitch.readme.html>
;; Author: Eduardo Ochs <eduardoochs@gmail.com>.
;; Version: 2012oct19
;; License: GPL3.

;; What is this
;; ============
;; Emacs modes like GUD and Slime use multiple windows and do not
;; expect that the current frame will have more windows than the ones
;; that they use - which means that they can't handle an "e-script
;; window" in the current frame...
;;
;; It SEEMS that if we craft very carefully window setups that include
;; all the windows that these modes use - so that they will not try to
;; create extra windows or rearrange the existing ones - then we can
;; also accomodate on the screen an e-script window, and things *MAY*
;; work.
;;
;; For example, these window configurations *should* work for GDB -
;; but the defaults for GUD have changed in ways that I don't
;; understand yet, so I can't be sure, I need more tests...
;;
;;     _________________________      ________________________ 
;;    |             |           |    |            |           |
;;    |   e-script  |           |    |  e-script  |  program  |
;;    |             |           |    |            |   source  |
;;    |_____________|  program  |    |____________|___________|
;;    |  GDB prompt |   source  |    |            |           |
;;    |     and     |           |    |    GDB     |  program  |
;;    | program I/O |           |    |   prompt   |    I/O    |
;;    |_____________|___________|    |____________|___________|
;;
;;              Old GUD                 New GUD, with gdb-mi
;;
;; The (obsolete) tests for the GDB interface have been moved to:
;;   (find-eev "eepitch.readme" "gdb-preparation")
;;   (find-eev "eepitch.readme" "gdb-3-windows")
;;   (find-eev "eepitch.readme" "gdb-4-windows")
;;
;; Note that it is also possible to run gdb via comint, in a
;; two-window setting, without any of the usuall GUD goodies...
;;   (find-es "anatocc" "stabs")
;;   (find-es "anatocc" "stabs" "eepitch-comint")






;; (find-angg ".emacs" "find-wset")
;; (find-angg ".emacs" "find-2")

(defun find-wset (chars &rest sexps)
  "Create a multi-window setting according to CHARS and SEXPS."
  (if (not (equal chars ""))
      (let ((c     (substring chars 0 1))
	    (crest (substring chars 1)))
	(cond ((equal c "1")
	       (delete-other-windows)
	       (apply 'find-wset crest sexps))
	      ((equal c "2")
	       (split-window-vertically)
	       (apply 'find-wset crest sexps))
	      ((equal c "3")
	       (split-window-horizontally)
	       (apply 'find-wset crest sexps))
	      ((equal c "o")
	       (other-window 1)
	       (apply 'find-wset crest sexps))
	      ((equal c "+")
	       (balance-windows)
	       (apply 'find-wset crest sexps))
	      ((equal c "_")
	       (eval (car sexps))
	       (apply 'find-wset crest (cdr sexps)))
	      (t (error))))))

(provide 'eev-multiwindow)





;; Local Variables:
;; coding:            raw-text-unix
;; ee-anchor-format:  "defun %s "
;; no-byte-compile:   t
;; End:
