;;; eev-tlinks.el --- hyperlinks to temporary buffers generated by templates

;; Copyright (C) 2012 Free Software Foundation, Inc.
;;
;; This file is not yet part of GNU eev.
;;
;; GNU eev is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; GNU eev is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.
;;
;; Author:     Eduardo Ochs <eduardoochs@gmail.com>
;; Maintainer: Eduardo Ochs <eduardoochs@gmail.com>
;; Version:    2012sep22
;; Keywords:   e-scripts, help, hyperlinks, hypertext

;;; Commentary:

;; The difference between the functions in this file and the ones in
;;   (find-eev "eev-elinks.el")
;; is subtle: 

; (find-efunctiondescr 'ee-upload-links)
;; (find-eev "eev-wrap.el" "eewrap-eewrap")


;; .find-find-links-links	(to "find-find-links-links")
;; .find-pdflikedef-links	(to "find-pdflikedef-links")
;;
;; .find-debpkg-links		(to "find-debpkg-links")
;; .find-eev-update-links	(to "find-eev-update-links")
;; .find-newhost-links	(to "find-newhost-links")
;; .find-dhmake-links		(to "find-dhmake-links")
;; .find-youtubedl-links	(to "find-youtubedl-links")
;; .find-upload-links		(to "find-upload-links")


(require 'eev-env)


;; Tools for generating defuns with `ee-template0'
;;
(defun ee-prepend-commas (str)
  (save-match-data
    (replace-regexp-in-string "\\([^ ]+\\)" ",\\1" str)))
(defun ee-if-nil-setq (str)
  (format "(setq %s (or %s \"{%s}\"))" str str str))
(defun ee-if-nil-setqs (vars sep)
  (mapconcat 'ee-if-nil-setq (save-match-data (ee-split vars)) sep))





;;;   __ _           _       _ _       _       /\ ____  
;;;  / _(_)_ __   __| |     | (_)_ __ | | ____|/\|___ \ 
;;; | |_| | '_ \ / _` |_____| | | '_ \| |/ / __|   __) |
;;; |  _| | | | | (_| |_____| | | | | |   <\__ \  / __/ 
;;; |_| |_|_| |_|\__,_|     |_|_|_| |_|_|\_\___/ |_____|
;;;                                                     
;;
;; find-find-links-links (to ".find-find-links-links")
;; See:
;; (find-eev "eev-template.el" "find-find-links-links")
;; (find-find-links-links "u" "find-links" "k stem args")

(defun find-find-links-links (&optional k stem args &rest pos-spec-list)
"Visit a temporary buffer containing hyperlinks for foo."
  (interactive)
  (setq k    (or k    "{k}"))
  (setq stem (or stem "{stem}"))
  (setq args (or args "{args}"))
  (apply 'find-elinks-elisp
   `((find-find-links-links ,k ,stem ,args ,@pos-spec-list)
     ;; Convention: the first sexp always regenerates the buffer.
     (find-efunction 'find-find-links-links)
     ""
     ,(ee-template0 "\
;; <find-{stem}-links>
;; {(ee-S `(find-find-links-links ,k ,stem ,args))}
;; A test: (find-{stem}-links ___)
\(define-key eev-mode-map \"\\M-h{k}\" 'find-{stem}-links)

\(defun find-{stem}-links (&optional {args} &rest pos-spec-list)
\"Visit a temporary buffer containing hyperlinks for foo.\"
  (interactive)
  {(ee-if-nil-setqs args \"\n  \")}
  (apply 'find-elinks
   `((find-{stem}-links {(ee-prepend-commas args)} ,@pos-spec-list)
     ;; Convention: the first sexp always regenerates the buffer.
     (find-efunction 'find-{stem}-links)
     \"\"
     ,@(ee-find-{stem}-links {args})
     )
   pos-spec-list))

\(defun ee-find-{stem}-links ({args})
  \"Return a list of sexps and strings for {stem}.
This is an internal function used by `find-{stem}-links'.\"
  `(
    ))")
     ) pos-spec-list))



;;;            _  __ _ _ _            _       __ 
;;;  _ __   __| |/ _| (_) | _____  __| | ___ / _|
;;; | '_ \ / _` | |_| | | |/ / _ \/ _` |/ _ \ |_ 
;;; | |_) | (_| |  _| | |   <  __/ (_| |  __/  _|
;;; | .__/ \__,_|_| |_|_|_|\_\___|\__,_|\___|_|  
;;; |_|                                          
;;
;; find-pdflikedef-links (to ".find-pdflikedef-links")
;; Used in: (find-eev "eev-pdflike.el")
;;    e.g.: (find-efunction 'find-xdvi-page)
;; (find-find-links-links "pdflikedef" "stem firstargs")
;;
(defun find-pdflikedef-links (&optional stem firstargs &rest rest)
  "Visit a temporary buffer containing hyperlinks for pdflikedef."
  (interactive)
  (setq stem (or stem "{stem}"))
  (setq firstargs (or firstargs "{firstargs}"))
  (apply 'find-elinks `(
    ;; Convention: the first sexp always regenerates the buffer.
    (find-pdflikedef-links ,stem ,firstargs ,@rest)
    (find-efunction 'find-pdflikedef-links)
    (emacs-lisp-mode)
    ;; Body:
    ""
    ,(ee-template `(stem ("c fname" ,firstargs)) "\
;; (find-efunction 'code-{stem})

;; find-{stem}page
;; find-{stem}-page
;; code-{stem}
;;
\(defalias 'find-{stem}page
          'find-{stem}-page)
\(defun     find-{stem}-page (fname &optional page &rest rest)
  (find-bgprocess (ee-find-{stem}-page fname page)))
\(defvar ee-find-{stem}-page-options '())
\(defun  ee-find-{stem}-page (fname &optional page)
  `(\"{stem}\"
    ,@ee-find-{stem}-page-options
    ,@(if page `(,(format \"--page=%d\" page)))
    ,fname))

\(defun      code-{stem} ({c fname} &rest rest)
  (eval (ee-read      (apply 'ee-code-{stem} {c fname} rest))))
\(defun find-code-{stem} ({c fname} &rest rest)
  (find-estring-elisp (apply 'ee-code-{stem} {c fname} rest)))
\(defun   ee-code-{stem} ({c fname} &rest rest)
  (concat (ee-template0 \"\\
\\(defun find-{c}page (&optional page &rest rest)
  (find-{stem}-page {(ee-pp0 fname)} page))
{(ee-code-pdftext-rest rest)}
\")  (ee-code-pdftext-rest rest)))

\(code-brfile 'find-xpdf-page :local 'brxpdfl :dired 'brxpdfl)


;; Tests:
;; (find-epp (ee-find-{stem}-page \"/tmp/foo.pdf\"))
;; (find-epp (ee-find-{stem}-page \"/tmp/foo.pdf\" 2))
;;           (find-{stem}-page    \"/tmp/foo.pdf\")
;;           (find-{stem}-page    \"/tmp/foo.pdf\" 2)
;;
;;      (find-code-{stem} \"foo\"   \"/tmp/foo.pdf\")
;;           (code-{stem} \"foo\"   \"/tmp/foo.pdf\")
;;                 (find-foopage)
;;                 (find-foopage 2)
;;      (code-pdftotext \"foo\"   \"/tmp/foo.pdf\")
")
    ) rest))




;;;   __ _           _           _      _           _         
;;;  / _(_)_ __   __| |       __| | ___| |__  _ __ | | ____ _ 
;;; | |_| | '_ \ / _` |_____ / _` |/ _ \ '_ \| '_ \| |/ / _` |
;;; |  _| | | | | (_| |_____| (_| |  __/ |_) | |_) |   < (_| |
;;; |_| |_|_| |_|\__,_|      \__,_|\___|_.__/| .__/|_|\_\__, |
;;;                                          |_|        |___/ 
;;
;; find-debpkg-links  (to ".find-debpkg-links")

(defun ee-links-for-debpkg (pkgname)
  "Return the three main links for the debian package PKGNAME."
  (list (ee-template1 `(pkgname) "\
{# }(find-status   \"{pkgname}\")
{# }(find-vldifile \"{pkgname}.list\")
{# }(find-udfile   \"{pkgname}/\")")))

(defun ee-dfs0 (pkg ext)
  "If the file /var/lib/dpkg/info/PKG.EXT exists, return a link to it."
  (let ((fname (concat pkg "." ext)))
    (if (file-exists-p (ee-vldifile fname))
	`(find-vldifile ,fname))))

(defun ee-links-for-debpkg-extra-vldi (pkg)
  "Return a list of links for files in /var/lib/dpkg/info/ belonging to PKG.
This is an internal function used by `find-debpkg-links'."
   (list (ee-dfs0 pkg "preinst")   (ee-dfs0 pkg "postinst")
	 (ee-dfs0 pkg "prerm")     (ee-dfs0 pkg "postrm")
	 (ee-dfs0 pkg "conffiles") (ee-dfs0 pkg "config")
	 (ee-dfs0 pkg "templates")
	 (ee-dfs0 pkg "md5sums")   (ee-dfs0 pkg "shlibs")
	 ))

(defun ee-debian-pooldir (pkg)
  "Used by `find-debpkg-links'; \"foo\" -> \"f\", \"libfoo\" -> \"libf\"."
  (if (string-match "^\\(lib\\)?." pkgname)
      (match-string 0 pkgname)))

(defun find-debpkg-links (&optional pkgname &rest rest)
  "Visit a temporary buffer containing hyperlinks related to a Debian package.
Try this: (find-debpkg-links \"bash\")"
  (interactive (list (ee-debpkgname-ask)))
  (setq pkgname (or pkgname "{pkgname}"))
  (apply 'find-elinks `(
    ;; Convention: the first sexp always regenerates the buffer.
    (find-debpkg-links ,pkgname ,@rest)
    (find-available ,pkgname)
    ""
    ,@(ee-links-for-debpkg pkgname)
    ""
    ,@(ee-links-for-debpkg-extra-vldi pkgname)
    ""
    ,(ee-template1 `(pkgname ("p" ,(ee-debian-pooldir pkgname))) "\
{# }(find-sh \"apt-cache dump | grep-dctrl -P {pkgname}\")
{# }(find-sh \"apt-cache search {pkgname} | sort\")
{# }(find-sh \"apt-cache showpkg {pkgname}\")
{# }(find-sh \"grep-aptavail -P {pkgname}\")

http://packages.debian.org/{pkgname}
http://packages.debian.org/src:{pkgname}
http://ftp.debian.org/debian/pool/main/{p}/{pkgname}/
http://backports.org/debian/pool/main/{p}/{pkgname}/
http://bugs.debian.org/cgi-bin/pkgreport.cgi?which=pkg&data={pkgname}&archive=no

http://packages.ubuntu.org/{pkgname}
")
    ) rest))



;;;   __ _           _           _          _           _ _     _ 
;;;  / _(_)_ __   __| |       __| |___  ___| |__  _   _(_) | __| |
;;; | |_| | '_ \ / _` |_____ / _` / __|/ __| '_ \| | | | | |/ _` |
;;; |  _| | | | | (_| |_____| (_| \__ \ (__| |_) | |_| | | | (_| |
;;; |_| |_|_| |_|\__,_|      \__,_|___/\___|_.__/ \__,_|_|_|\__,_|
;;;                                                               
;; This is a VERY EARLY prototype (hi Marc!)
;; of a rewrite of something that was very ugly.

;; (find-angg ".emacs" "find-dpkg-build-escript")

(defun ee-dsc-url-split (dsc-url)
  "Example:
  (ee-dsc-url-split
   \"http://ftp.debian.org/debian/pool/main/i/imagemagick/imagemagick_6.2.4.5.dfsg1-0.9.dsc\")
  -> (\"http\" \"ftp.debian.org/debian/pool/main/i/imagemagick/\"
      \"imagemagick\" \"6.2.4.5.dfsg1\" \"-0.9\")"
  (let ((prot://dir/ (file-name-directory dsc-url))
	(fname-dsc (file-name-nondirectory dsc-url))
	prot dir/ xxx vvv -sv)
    (if (string-match "^\\([a-z]+\\)://\\(.*\\)" prot://dir/)
	(setq prot (match-string 1 prot://dir/)
	      dir/ (match-string 2 prot://dir/)))
    (if (string-match "^\\([^_]+\\)_\\([^-]+\\)\\(-.*\\)?\\.dsc$" fname-dsc)
	(setq xxx (match-string 1 fname-dsc)
	      vvv (match-string 2 fname-dsc)
	      -sv (or (match-string 3 fname-dsc) "")))
    (list prot dir/ xxx vvv -sv)))

(defun ee-links-for-dscbuild (dsc-url)
  (apply 'ee-links-for-dscbuild0
	 (downcase (format-time-string "%Y%b%d"))
	 (ee-dsc-url-split dsc-url)))

(defun ee-links-for-dscbuild0 (date prot dir/ xxx vvv -sv)
  (ee-template
   '(date prot dir/ xxx vvv -sv) "\
#####
#
# {xxx} (from the debian sources)
# {date}
#
#####

# <{xxx}-deb-src>
# {prot}://{dir/}
# {prot}://{dir/}{xxx}_{vvv}{-sv}.dsc
# {prot}://{dir/}{xxx}_{vvv}{-sv}.diff.gz
# {prot}://{dir/}{xxx}_{vvv}.orig.tar.gz
#
rm -Rv ~/usrc/{xxx}/
mkdir  ~/usrc/{xxx}/
cd $S/{prot}/{dir/}
cp -v {xxx}_{vvv}* ~/usrc/{xxx}/
cd     ~/usrc/{xxx}/
dpkg-source -sn -x {xxx}_{vvv}{-sv}.dsc
cd     ~/usrc/{xxx}/{xxx}-{vvv}/
dpkg-buildpackage -us -uc -b -rfakeroot     2>&1 | tee odb

#
# (find-fline \"~/usrc/{xxx}/\")
 (eepitch-shell)
cd ~/usrc/{xxx}/
sudo dpkg -i *.deb

#
# (code-c-d \"{xxx}\" \"~/usrc/{xxx}/{xxx}-{vvv}/\")
# (find-{xxx}file \"\")"))



;;;                                       _       _       
;;;   ___  _____   __     _   _ _ __   __| | __ _| |_ ___ 
;;;  / _ \/ _ \ \ / /____| | | | '_ \ / _` |/ _` | __/ _ \
;;; |  __/  __/\ V /_____| |_| | |_) | (_| | (_| | ||  __/
;;;  \___|\___| \_/       \__,_| .__/ \__,_|\__,_|\__\___|
;;;                            |_|                        
;;
;; find-eev-update-links (to ".find-eev-update-links")
;; (find-eev-update-links)
;; (find-find-links-links "eev-update" "dir")

(defun find-eev-update-links (&optional dir &rest rest)
  "Visit a temporary buffer containing hyperlinks for eev-update.
Warning (2012): I haven't used this in ages!"
  (interactive)
  (setq dir (or dir "{dir}"))
  (let ((edir (or dir (ee-expand ee-eevdir))))
    (apply 'find-elinks `(
     ;; Convention: the first sexp always regenerates the buffer.
     (find-eev-update-links ,dir ,@rest)
     (find-eev-update-links ,dir ,@rest)
     (find-eev-update-links "~/eev-current/" ,@rest)
     (find-eev-update-links ,edir ,@rest)
     (find-efunction 'find-eev-update-links)
     ""
     (ee-expand ee-eevdir)
     (find-eevfile "eev.el" "$EEVDIR")
     (find-eevfile "")
     ""
     ,(ee-template `(dir edir) "\
 (eepitch-shell)
 (eepitch-kill)
 (eepitch-shell)

mkdir {edir}
cd    {edir}
mv -v eev-current.tar.gz eev-current-old.tar.gz
wget http://angg.twu.net/eev-current.tar.gz
tar -xvzf eev-current.tar.gz

\(progn
  (add-to-list 'load-path \"{edir}\")
  (require 'eev-all)
  (eev-mode 1)
  )

 (eepitch-shell)
 (eepitch-kill)
 (eepitch-shell)

cd {dir}
./eev-rctool prepare
./eev-rctool install_rc")
     ""
     (find-eevfile "eev-rctool")
     (find-eevsh "./eev-rctool help")
     (find-eevsh "./eev-rctool notes")
     (find-eevtmpfile "")
     (find-eevtmpfile "backup/")
     (find-eevtmpfile "backup/do_install.sh")
     (find-eevtmpfile "backup/do_uninstall.sh")
     (find-fline "~/.emacs" "Beginning of the eev block:")
     ) rest)))









;;;                      _               _   
;;;  _ __   _____      _| |__   ___  ___| |_ 
;;; | '_ \ / _ \ \ /\ / / '_ \ / _ \/ __| __|
;;; | | | |  __/\ V  V /| | | | (_) \__ \ |_ 
;;; |_| |_|\___| \_/\_/ |_| |_|\___/|___/\__|
;;;                                          
;; find-newhost-links  (to ".find-newhost-links")
;; Scripts to connect to a newly-installed Debian machine
;; (find-angg ".emacs.templates" "find-newhost-links")
;; (find-find-links-links "newhost" "thatmname thatip thismname thisip thisiface")
;; (find-angg "bin/etc.lua" "inet_addr")

(defvar ee-this-mname nil)
(defvar ee-this-iface "wlan0")
(defvar ee-this-ip nil)
;;
(defun  ee-this-mname () (find-sh0 "uname -n"))
(defun  ee-this-ip    (&optional iface)
  (find-sh0 (format "/sbin/ifconfig %s | etc.lua inet_addr"
		    (or ee-this-iface iface))))
;;
;;   (find-sh0 "uname -n")
;;   (find-sh0 "/sbin/ifconfig")
;;   (find-sh0 "/sbin/ifconfig wlan0")
;;   (find-sh0 "/sbin/ifconfig eth0")
;;   (setq ee-this-iface "wlan0")
;;   (setq ee-this-iface "eth0")
;;   (ee-this-mname)
;;   (ee-this-ip)
;; (setq ee-this-mname (ee-this-mname))
;; (setq ee-this-ip (ee-this-ip))

(defun find-newhost-links (&optional thatmname thatip thismname thisip thisiface &rest rest)
  "Visit a temporary buffer with an e-script to set up a new host."
  (interactive)
  (let ((thisdisplay))
    (setq thatmname (or thatmname "{thatmname}"))
    (setq thatip    (or thatip    "{thatip}"))
    (setq thismname (or thismname ee-this-mname "{thismname}"))
    (setq thisip    (or thisip    ee-this-ip    "{thisip}"))
    (setq thisiface (or thisiface ee-this-iface "{thisiface}"))
    (setq thisdisplay (or (getenv "DISPLAY") "{thisdisplay}"))
    (apply 'find-elinks `(
      ;; Convention: the first sexp always regenerates the buffer.
      (find-newhost-links ,thatmname ,thatip ,thismname ,thisip ,thisiface ,@rest)
      (find-newhost-links ,thatmname ,thatip ,ee-this-mname ,ee-this-ip ,ee-this-iface ,@rest)
      (find-newhost-links ,thatmname ,thatip nil nil nil ,@rest)
      (find-newhost-links ,thatmname ,thatip)
      (find-efunction 'find-newhost-links)
      ;; Body:
      ""
      ,(ee-template `(thatmname thatip
		      thismname thisip thisiface
		      thisdisplay) "\
# Basic setup (on this machine, {thismname} - set thismname and thisip):
#   (find-sh0 \"/sbin/ifconfig wlan0\")
#   (find-sh0 \"/sbin/ifconfig eth0\")
#   (setq ee-this-iface \"wlan0\")
#   (setq ee-this-iface \"eth0\")
#   (ee-this-mname)
#   (ee-this-ip)
# (setq ee-this-mname (ee-this-mname))
# (setq ee-this-ip    (ee-this-ip))
# (find-newhost-links \"{thatmname}\" \"{thatip}\")

# Make sure that we can refer to {thatmname} by name
#   (find-sh0 \"ls -l          /etc/hosts\")
#   (find-sh0 \"sudo chmod 666 /etc/hosts\")
#   (kill-new \"{thatip}   {thatmname}\")
#   (find-fline \"/etc/hosts\")
#   (find-fline \"/etc/hosts\" \"{thatmname}\")
#   (find-fline \"/etc/hosts\" \"{thatip}\")

# Basic setup (on the remote machine, {thatmname} - by hand):
#   adduser edrx
#   (find-es \"sudo\" \"sudo\")
#   apt-get install openssh-server xterm
#   chmod 666 /etc/hosts
#   nano      /etc/hosts
#   (kill-new \"{thisip}   {thismname}\")

# Try to connect:
#   (find-sh0 \"ssh-keygen -R {thatip}\")
#   (find-sh0 \"ssh-keygen -R {thatmname}\")

 (eepitch-shell)
 (eepitch-kill)
 (eepitch-shell)
ssh edrx@{thatip}
exit
ssh edrx@{thatmname}
exit

# Try file access by tramp
#   (find-fline \"/ssh:edrx@{thatip}:/\")
#   (find-fline \"/scp:edrx@{thatip}:/\")
#   (find-fline \"/ssh:edrx@{thatmname}:/\")
#   (find-fline \"/scp:edrx@{thatmname}:/\")

;; If that works:
\(code-c-d \"{thatmname}\" \"/scp:edrx@{thatmname}:\")
\(code-c-d \"{thatmname}\" \"/ssh:edrx@{thatmname}:\")
;; (find-{thatmname}file \"/\")
;; (find-{thatmname}file \"/home/edrx/\")
\(defun eepitch-{thatmname} () (interactive)
  (eepitch '(find-comintprocess \"ssh {thatmname}\" \"ssh edrx@{thamname}\")))


# Make sure that the remote machine knows this hostname
ssh edrx@{thatmname}
  #
  # Does the remote machine know this hostname?
  echo \"{thisip}   {thismname}\"
  cat            /etc/hosts | grep {thismname}
  #
  # If it isn't there, add it:
  cat            /etc/hosts
  ls -l          /etc/hosts
  sudo chmod 666 /etc/hosts
  echo \"{thisip}   {thismname}\" >> /etc/hosts
  cat            /etc/hosts | grep {thismname}
  cat            /etc/hosts
  #
  # Try to open a remote xterm on this X server
  # (find-sh0 \"xhost +\")
  DISPLAY={thismname}{thisdisplay} xterm -T \"xterm@{thatmname}\" &
  exit


# A high-level version of the above:
# (find-sh0 \"xhost +\")
# (find-{thatmname}file \"/etc/hosts\" \"{thismname}\")
#   (eekill \"{thisip}   {thismname}\\n\")
#   (eekill \"# (setq backup-inhibited t)\\n\")

 (eepitch-{thatmname})
 (eepitch-kill)
 (eepitch-{thatmname})
DISPLAY={thismname}{thisdisplay} xterm -T \"xterm@{thatmname}\" &

")
    ) rest)))

;; (find-newhost-links)
;; (find-newhost-links "gwen" "192.168.1.101")



;;;      _ _                     _        
;;;   __| | |__  _ __ ___   __ _| | _____ 
;;;  / _` | '_ \| '_ ` _ \ / _` | |/ / _ \
;;; | (_| | | | | | | | | | (_| |   <  __/
;;;  \__,_|_| |_|_| |_| |_|\__,_|_|\_\___|
;;;                                       
;; find-dhmake-links  (to ".find-dhmake-links")
;;
;; Some templates for generating ".deb"s.
;; If you want to build a .deb for a package called, say, "foo-bar",
;; these scripts will use the directory "~/usrc/foo-bar/foo-bar_xxx/"
;; to build it - that directory will be recreated from scratch each
;; time - and the "override files for dhmake" will be taken from the
;; directory "~/foo-bar/". At this moment these paths are hardcoded.
;;
;; I generate the 3 main packages at
;;   http://angg.twu.net/debian/
;;   http://angg.twu.net/debian/README.html
;; with these commands:
;;   (find-dhmake-links "eev" "eevbuild")
;;   (find-dhmake-links "eev-puro" "eevpuro")
;;   (find-dhmake-links "eev-lua-extras" "eevluaextras")

(defvar ee-dhmake-fullname "Eduardo Ochs")
(defvar ee-dhmake-email "eduardoochs@gmail.com")

;; (find-estring-elisp (ee-dhmake-codecds "eev-puro" "eevpuro" "20120305"))
;;
(defun ee-dhmake-codecds (stem c date)
  (ee-template '(stem c date) "\
;; Generated by:
;; (ee-dhmake-codecds \"{stem}\" \"{c}\" \"{date}\")
;;
\(code-c-d \"{c}dh\"  \"~/{stem}/\")
\(code-c-d \"{c}dhd\" \"~/{stem}/debian/\")
\(code-c-d \"{c}\"    \"~/usrc/{stem}/{stem}-0.1.{date}/\")
\(code-c-d \"{c}d\"   \"~/usrc/{stem}/{stem}-0.1.{date}/debian/\")
\(code-c-d \"{c}dd\"  \"~/usrc/{stem}/{stem}-0.1.{date}/debian/{stem}/\")
\(code-c-d \"{c}unp\" \"~/usrc/{stem}/{stem}-unpacked/\")
;; (find-{c}dhfile  \"\")
;; (find-{c}dhdfile \"\")
;; (find-{c}file    \"\")
;; (find-{c}dfile   \"\")
;; (find-{c}ddfile  \"\")
;; (find-{c}unpfile \"\")
;; (find-{c}dsh   \"find * | sort\")
;; (find-{c}ddsh  \"find * | sort\")
;; (find-{c}unpsh \"find * | sort\")
;; (find-status   \"{stem}-deb\")
;; (find-vldifile \"{stem}-deb.list\")
;; (find-udfile   \"{stem}-deb-puro/\")"))

;; (find-estring (ee-dhmake-build "eev-puro" "eevpuro" "20120305"))
;;
(defun ee-dhmake-build (stem c date)
  (ee-template '(stem c date ee-dhmake-fullname ee-dhmake-email) "\
# Generated by:
# (ee-dhmake-build \"{stem}\" \"{c}\" \"{date}\")
#
rm   -Rv ~/usrc/{stem}/
mkdir -p ~/usrc/{stem}/
mkdir    ~/usrc/{stem}/{stem}-0.1.{date}/
# ln -s                {stem}-0.1.{date}  ~/usrc/{stem}/{stem}-0.1
  ln -s                {stem}-0.1.{date}  {stem}-0.1
cd       ~/usrc/{stem}/{stem}-0.1.{date}/

echo | \\
DEBFULLNAME=\"{ee-dhmake-fullname}\" \\
  dh_make --email {ee-dhmake-email} \\
          --copyright=gpl \\
          --cdbs \\
          --native
rm -v debian/README.Debian
rm -v debian/*.EX
rm -v debian/*.ex
cp -iv debian/control debian/control.orig

# (find-fline      \"~/{stem}/debian/\")
# (find-sh0 \"rm -Rv ~/{stem}/debian/\")
mkdir  -p ~/{stem}/debian/
if [ ! -e ~/{stem}/Makefile ]; then
  echo Creating:               ~/{stem}/Makefile
  echo \"clean:\"              > ~/{stem}/Makefile
fi
if [ ! -e ~/{stem}/debian/control ]; then
  echo Creating:               ~/{stem}/debian/control
  cp -v debian/control         ~/{stem}/debian/control
fi
if [ ! -e ~/{stem}/debian/rules ]; then
  echo Creating:               ~/{stem}/debian/rules
  cp -v debian/rules           ~/{stem}/debian/rules
  echo \"build/{stem}::\"   >> ~/{stem}/debian/rules
  echo \"install/{stem}::\" >> ~/{stem}/debian/rules
fi

# (find-{c}dh \"debian/\")
# (find-{c}dh \"debian/control\")
# (find-{c}dh \"debian/rules\")
cp -v ~/{stem}/debian/* debian/
cp -v ~/{stem}/Makefile .

# (find-man \"1 dpkg-buildpackage\")
# dpkg-buildpackage -us -uc    -rfakeroot     2>&1 | tee odb
  dpkg-buildpackage -us -uc -b -rfakeroot     2>&1 | tee odb
# (find-{c}file \"odb\")

rm -Rv ~/usrc/{stem}/{stem}-unpacked/
mkdir  ~/usrc/{stem}/{stem}-unpacked/
mkdir  ~/usrc/{stem}/{stem}-unpacked/DEBIAN/
cd     ~/usrc/{stem}/
ar p {stem}_*.deb control.tar.gz | tar -C {stem}-unpacked/DEBIAN/ -xvzf -
ar p {stem}_*.deb data.tar.gz    | tar -C {stem}-unpacked/        -xvzf -

# (find-{c}unpfile \"\")
# (find-{c}unpfile \"DEBIAN/\")
# (find-{c}unpfile \"DEBIAN/control\")
# (find-{c}unpsh \"find * | sort\")"))

(defun find-dhmake-links (&optional stem c date &rest rest)
  "Visit a temporary buffer containing hyperlinks for dhmake."
  (interactive)
  (setq stem (or stem "{stem}"))
  (setq c    (or c    "{c}"))
  (setq date (or date (format-time-string "%Y%m%d")))
  (apply 'find-elinks `(
    ;; Convention: the first sexp always regenerates the buffer.
    (find-dhmake-links ,stem ,c ,date ,@rest)
    (find-efunction 'find-dhmake-links)
    ;; Body:
    ""
    ;; (find-estring-elisp (ee-dhmake-codecds ,stem ,c ,date))
    ;; (find-estring (ee-dhmake-build ,stem ,c ,date))
    ;; (eev (ee-dhmake-build ,stem ,c ,date))
    ;; (find-fline "$EE")
    ;; (find-fline (format "~/%s/debian/" stem))
    ;; (find-sh0 ,(format "rm -Rv ~/%s/debian/" stem))
    ;; ""
    ,(ee-template `(stem c date) "\
# Set up the build script that `ee' will execute:
# (find-estring-elisp (ee-dhmake-codecds \"{stem}\" \"{c}\" \"{date}\"))
# (find-estring       (ee-dhmake-build   \"{stem}\" \"{c}\" \"{date}\"))
# (eev                (ee-dhmake-build   \"{stem}\" \"{c}\" \"{date}\"))
# (find-fline \"$EE\")

# Examine its main control files:
# (find-fline \"~/{stem}/debian/rules\")
# (find-fline \"~/{stem}/debian/control\")
# (find-fline \"~/{stem}/debian/\")

 (eepitch-shell)
 (eepitch-kill)
 (eepitch-shell)
ee
cd ~/usrc/{stem}/
ls -lF *.deb
sudo dpkg -i {stem}*.deb
# sudo dselect update

# (find-status   \"{stem}\")
# (find-vldifile \"{stem}.list\")
# (find-udfile   \"{stem}/\")


# Upload to angg.twu.net.
# (THIS IS A HACK!)
# (find-twusfile \"debian/\")

 (eepitch-eshell)
cd ~/usrc/{stem}/
ls -l {stem}_0.1.{date}_all.deb
cp -v {stem}_0.1.{date}_all.deb (ee-twusfile \"debian/\")
ls -l (ee-twusfile \"debian/\")
#   (find-twusfile \"debian/\")

 (eepitch-Twu)
cd ~/slow_html/debian/
mkdir -p dists/./main/binary-i386/
mkdir -p dists/./main/binary-amd64/
ls -lAF
apt-ftparchive packages .
apt-ftparchive packages . \\
  | tee        dists/./main/binary-i386/Packages \\
  | gzip -c9 > dists/./main/binary-i386/Packages.gz
apt-ftparchive packages . \\
  | tee        dists/./main/binary-amd64/Packages \\
  | gzip -c9 > dists/./main/binary-amd64/Packages.gz
")
    ) rest))
 
;; (find-dhmake-links)
;; (find-dhmake-links "{stem}" "{c}" "{date}")
;; (find-dhmake-links "foo-bar" "foobar")
;; (find-dhmake-links "eev" "eevbuild")
;; (find-dhmake-links "eev" "eevbuild" "20120404")





;;;                    _         _                    _ _ 
;;;  _   _  ___  _   _| |_ _   _| |__   ___        __| | |
;;; | | | |/ _ \| | | | __| | | | '_ \ / _ \_____ / _` | |
;;; | |_| | (_) | |_| | |_| |_| | |_) |  __/_____| (_| | |
;;;  \__, |\___/ \__,_|\__|\__,_|_.__/ \___|      \__,_|_|
;;;  |___/                                                
;;
;; find-youtubedl-links  (to ".find-youtubedl-links")
;; (find-angg ".emacs.templates" "find-youtubedl-links")

(defvar ee-youtubedl-dir "/sda5/videos/")

(defun find-youtubedl-links (&optional dir title hash ext stem &rest rest)
  "Visit a temporary buffer containing hyperlinks for youtube-dl."
  (interactive)
  (setq dir   (or dir "{dir}"))
  (setq title (or title "{title}"))
  (setq hash  (or hash "{hash}"))
  (setq ext   (or ext "{ext}"))
  (setq stem  (or stem "{stem}"))
  (apply 'find-elinks `(
    ;; Convention: the first sexp always regenerates the buffer.
    (find-youtubedl-links ,dir ,title ,hash ,ext ,stem ,@rest)
    (find-youtubedl-links ee-youtubedl-dir ,title ,hash ,ext ,stem ,@rest)
    (setq ee-youtubedl-dir ,ee-youtubedl-dir)
    ;; Body:
    ""
    ,(ee-template `(dir title hash ext stem) "\
 (eepitch-shell2)
 (eepitch-kill)
 (eepitch-shell2)
# http://www.youtube.com/watch?v={hash}
cd {dir}
youtube-dl -t 'http://www.youtube.com/watch?v={hash}'

# (find-es \"video\" \"youtube-dl\")
# (find-fline \"{dir}\" \"{hash}\")
# (find-fline \"{dir}\" \"{title}-{hash}\")
# (find-fline \"{dir}\" \"{title}-{hash}{ext}\")
# (find-mplayer  \"{dir}{title}-{hash}{ext}\")
# (find-mplayer  \"{dir}{title}-{hash}{ext}.part\")
# (code-mplayer \"{stem}video\" \"{dir}{title}-{hash}{ext}\")
# (code-mplayer \"{stem}video\" \"{dir}{title}-{hash}{ext}.part\")
# (find-{stem}video)
")
    ) rest))



;;;  _   _       _                 _                   _ 
;;; | | | |_ __ | | ___   __ _  __| |   __ _ _ __   __| |
;;; | | | | '_ \| |/ _ \ / _` |/ _` |  / _` | '_ \ / _` |
;;; | |_| | |_) | | (_) | (_| | (_| | | (_| | | | | (_| |
;;;  \___/| .__/|_|\___/ \__,_|\__,_|  \__,_|_| |_|\__,_|
;;;       |_|                                            
;;;      _                     _                 _ 
;;;   __| | _____      ___ __ | | ___   __ _  __| |
;;;  / _` |/ _ \ \ /\ / / '_ \| |/ _ \ / _` |/ _` |
;;; | (_| | (_) \ V  V /| | | | | (_) | (_| | (_| |
;;;  \__,_|\___/ \_/\_/ |_| |_|_|\___/ \__,_|\__,_|
;;;                                                
;; find-upload-links  (to ".find-upload-links")

;; 2012jan26 - experimental hack.
;; Some of these functions use environment variables for readability
;; (i.e., to make the shell scripts more readable).
;; Default values (all for angg.twu.net):
(ee-setenv "MYPAGEDIR" "/scp:edrx@angg.twu.net:public_html")
(ee-setenv "MYPAGEURL" "http://angg.twu.net")
(ee-setenv "DNPAGEURL" "http://angg.twu.net")
;; (getenv "MYPAGEDIR")
;; (getenv "MYPAGEURL")
;; (getenv "DNPAGEURL")
;; (setenv "DNPAGEURL" "http://0branch.com/highlight/snippets")

(defun find-tkdiff (f1 f2)
  (find-bgprocess `("tkdiff" ,f1 ,f2)))

(defun ee-upload-links (fromdir/ todir/ fname)
  "An internal function used by `find-upload-links'.
Try this: (find-elinks (ee-upload-links \"eev-current/eev-template.el\"))"
  (let ((dir    (file-name-directory (ee-expand fname)))
	(fname- (file-name-nondirectory fname))
	(mypagedir (getenv "MYPAGEDIR"))
	(mypageurl (getenv "MYPAGEURL"))
	)
    `("# Env vars (current values):"
      (setenv "MYPAGEDIR" ,(getenv "MYPAGEDIR"))
      (setenv "MYPAGEURL" ,(getenv "MYPAGEURL"))
      ,(ee-template1 `(fromdir/ todir/ fname dir fname- mypagedir mypageurl) "\
# Upload (warning: SLOW, uses tramp!):
 (eepitch-eshell)
cp -v ~/{fromdir/}{fname} $MYPAGEDIR/{todir/}{fname}
ls -l ~/{fromdir/}{fname} $MYPAGEDIR/{todir/}{fname}\n
# Test:
 (eepitch-shell)
 (eepitch-kill)
 (eepitch-shell)
cd /tmp/
wget -N {mypageurl}/{fname}
ls -l /tmp/{fname-} ~/{fname}
# (find-fline \"/tmp/{fname-}\")"))))

(defun ee-download-links (fromdir/ todir/ fname)
  "Visit a temporary buffer containing a script for downloading FNAME."
  (setq fromdir/ (or fromdir/ (file-name-directory fname)))
  (setq todir/   (or todir/   (file-name-directory fname)))
  (let ((fname-               (file-name-nondirectory fname))
	(dnpageurl            (getenv "DNPAGEURL")))
    `(,(ee-template `(fromdir/ todir/ fname fname- dnpageurl) "\
# Download:
 (eepitch-shell)
 (eepitch-kill)
 (eepitch-shell)
cd /tmp/
wget -N {dnpageurl}/{fromdir/}{fname}
ls -l /tmp/{fname-} ~/{todir/}{fname}
cp -v /tmp/{fname-} ~/{todir/}{fname}
#        (diff \"/tmp/{fname-}\" \"~/{todir/}{fname}\")
# (find-tkdiff \"/tmp/{fname-}\" \"~/{todir/}{fname}\")
# (find-fline \"/tmp/{fname-}\")
# (find-fline \"~/{todir/}{fname}\")"))))

(defun find-upload-links (&optional fromdir/ todir/ fname &rest rest)
  "Visit a temporary buffer containing a script for uploading FNAME."
  (interactive)
  (setq fname (or fname "{fname}"))
  (setq fromdir/ (or fromdir/ (file-name-directory fname)))
  (setq todir/   (or todir/   (file-name-directory fname)))
  (let ((ee-hyperlink-prefix "# "))
    (apply 'find-elinks `(
      ;; Convention: the first sexp always regenerates the buffer.
      (find-upload-links ,fromdir/ ,todir/ ,fname ,@rest)
      ;; The second sexp generates the corresponding download link.
      (find-download-links ,fname ,@rest)
      ""
      ,@(ee-upload-links fromdir/ todir/ fname)) rest)))

(defun find-download-links (&optional fromdir/ todir/ fname extras &rest rest)
  "Visit a temporary buffer containing a script for downloading FNAME."
  (interactive)
  (setq fname    (or fname "{fname}"))
  (setq fromdir/ (or fromdir/ (file-name-directory fname)))
  (setq todir/   (or todir/   (file-name-directory fname)))
  (apply 'find-elinks `(
    ;; Convention: the first sexp always regenerates the buffer.
    (find-download-links ,fromdir/ ,todir/ ,fname ,@rest)
    ;; Body:
    ""
    ,@(ee-download-links fromdir/ todir/ fname)
    ,@extras) rest))

(defun ut ()
  (interactive)
  "Upload eev-template.el"
  (find-upload-links "eev-current/" "eev-current/" "eev-template.el"))

(defun dt ()
  (interactive)
  "Download eev-template.el and load the new version."
  (find-download-links
   "eev-current/" "emacs/eev/" "eev-template.el"
   '("\n (load \"eev-template.el\")")))






(provide 'eev-tlinks)


;; was: ee-anchor-format:  "defun %s "



;; Local Variables:
;; coding:            raw-text-unix
;; ee-anchor-format:  "%s"
;; no-byte-compile:   t
;; End:
